package tumbler.internal.domain;

import org.junit.runner.*;

public class ScenarioModel {

	public enum ScenarioStatus {
		PASSED, FAILED, PENDING
	}

	private String name;
	private String given;
	private String when;
	private String then;
	private StoryModel story;
	private ScenarioStatus status;
	private Description description;

	public ScenarioModel(String name) {
		this.name = name.trim();
	}

	public String name() {
		return name;
	}

	public String given() {
		return processStory(given);
	}

	public String when() {
		return processStory(when);
	}

	public String then() {
		return processStory(then);
	}

	private String processStory(String text) {
		if (text == null)
			return "";

		text = text.replaceAll("\\$it", story.name());
		return text;
	}

	public ScenarioModel withName(String name) {
		this.name = name.trim();
		return this;
	}

	public ScenarioModel withGiven(String given) {
		this.given = given;
		return this;
	}

	public ScenarioModel withWhen(String when) {
		this.when = when;
		return this;
	}

	public ScenarioModel withThen(String then) {
		this.then = then;
		return this;
	}

	public ScenarioModel withStatus(ScenarioStatus status) {
		this.status = status;
		return this;
	}

	public ScenarioStatus status() {
		return status;
	}

	@Override
	public boolean equals(Object obj) {
		ScenarioModel other = (ScenarioModel) obj;
		if (other == null)
			return false;
		if ((other.name == null && name != null) || (name == null && other.name != null))
			return false;
		if (other.name.toLowerCase().equals(name.toLowerCase()))
			return true;
		return false;
	}

	void setStory(StoryModel story) {
		this.story = story;
	}

	@Override
	public String toString() {
		return name;
	}

	public ScenarioModel withDescription(Description description) {
		this.description = description;
		return this;
	}

	public Description description() {
		return description;
	}

}
