package tumbler.internal.writers;

import java.io.*;
import java.util.*;

import tumbler.internal.domain.*;

public abstract class StoryFileWriter {

	public static final String REPORT_DIR = "Tumbler-reports";

	protected String fileExtension;
	protected String tocFile;
	protected String outputFolder;

	public void write(StoryModel story) {
		String text = textFor(story);
		try {
			writeTextToFile(story, text);
		} catch (Exception e) {
			throw new RuntimeException("File could not be generated.", e);
		}
	}

	abstract String textFor(StoryModel story);

	public void writeToc(List<StoryModel> storiesList) {
		String text = tocFor(storiesList);
		try {
			writeTocToFile(text);
		} catch (Exception e) {
			throw new RuntimeException("File could not be generated.", e);
		}
	}

	protected void writeTextToFile(StoryModel story, String text) throws IOException {
		createOutputFolder();
		String fullDir = createPackageFolders(story.packageName());
		FileWriter writer = new FileWriter(new File(fullDir + File.separator + story.camelisedName() + fileExtension), false);
		writer.append(text);
		writer.flush();
		writer.close();
	}

	protected void writeTocToFile(String text) throws IOException {
		FileWriter writer = new FileWriter(new File(outputFolder + File.separator + tocFile), false);
		writer.append(text);
		writer.flush();
		writer.close();
	}

	protected abstract String tocFor(List<StoryModel> storiesList);

	private String createPackageFolders(String packageName) {
		if ("".equals(packageName))
			return outputFolder;

		packageName = packageName.replaceAll("\\.", File.separator);
		String fullDir = outputFolder + File.separator + packageName;
		new File(fullDir).mkdirs();
		return fullDir;
	}

	private String createOutputFolder() {
		loadOutputFolderFromProperty();
		outputFolder += File.separator + REPORT_DIR;
		executeCreateOutputFolder();
		return outputFolder;
	}

	private void executeCreateOutputFolder() {
		if (!new File(outputFolder).exists())
			new File(outputFolder).mkdirs();
	}

	private void loadOutputFolderFromProperty() {
		outputFolder = System.getProperty("outputFolder");
		if (outputFolder == null) {
			outputFolder = "target";
			System.setProperty("outputFolder", outputFolder);
		}
	}

	protected void copy(InputStream in, File dst) throws IOException {
		OutputStream out = new FileOutputStream(dst);

		byte[] buf = new byte[1024];
		int len;
		while ((len = in.read(buf)) > 0) {
			out.write(buf, 0, len);
		}

		in.close();
		out.close();
	}
}